/*
	The kernel estimation class.

	A kernel density estimate is constructed with the given points and bandwidth (and the Gaussian kernel).
	It is automatically normalized to zero mean and unit variance.
*/

class StandardizedKernelEstimator {
	decl points, bandwidth;
	StandardizedKernelEstimator(const points, const bandwidth);
	dens(const x);
	ddens(const x);
	dlog(const x);
};

/*
	Construct the standardized kernel estimate.

	points: 1xT vector of data points
	bandwidth: pre-normalization bandwidth
*/
StandardizedKernelEstimator::StandardizedKernelEstimator(const points, const bandwidth) {
	decl mixturemean = meanr(points);
	decl mixturesd = sqrt(meanr((points-mixturemean).^2 + bandwidth^2));
	this.points = (points - mixturemean) / mixturesd;
	this.bandwidth = bandwidth / mixturesd;
}

/*
	Compute the density estimate.

	x: 1xn vector of values at which to evaluate the density
*/
StandardizedKernelEstimator::dens(const x) {
	return meanr(densn((x' - points) / bandwidth) / bandwidth)';
}

/*
	Compute the derivative of the density.

	x: 1xn vector values at which to evaluate the derivative
*/
StandardizedKernelEstimator::ddens(const x) {
	return meanr(-densn((x' - points) / bandwidth) / bandwidth^3 .* (x' - points))';
}

/*
	Compute the derivative of the logarithm of the density (used for SP-GAS).

	x: 1xn vector values at which to evaluate the derivative
*/
StandardizedKernelEstimator::dlog(const x) {
	return ddens(x) ./ dens(x);
}